--
-- Database design for the monitor_gui database
--

DROP TABLE IF EXISTS gui_action_log;
CREATE TABLE gui_action_log(
	user			VARCHAR(30) NOT NULL,
	action			VARCHAR(50) NOT NULL,
	time			TIMESTAMP
) ;

DROP TABLE IF EXISTS gui_access;
CREATE TABLE gui_access(
	user			VARCHAR(30) NOT NULL UNIQUE PRIMARY KEY,
	view			BOOL,
	view_all		BOOL,
	modify_obj		BOOL,
	modify_any		BOOL,
	delete_obj		BOOL,
	delete_all		BOOL,
	import			BOOL,
	probe			BOOL,
	admin			BOOL
) ;
INSERT INTO gui_access VALUES('monitor', 1, 1, 1, 1, 1, 1, 1, 1, 1);


DROP TABLE IF EXISTS contact;
CREATE TABLE contact(
	id								    INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	contact_name					    VARCHAR(75),
	alias							    VARCHAR(160) NOT NULL,
	host_notifications_enabled			BOOL,
	service_notifications_enabled		BOOL,
	can_submit_commands					BOOL,
	retain_status_information			BOOL,
	retain_nonstatus_information		BOOL,
	host_notification_period		    INT,
	service_notification_period		    INT,
	host_notification_options		    VARCHAR(15),
	service_notification_options	    VARCHAR(15),
	host_notification_commands		    INT,
	host_notification_commands_args	    TEXT,
	service_notification_commands       INT,
	service_notification_commands_args  TEXT,
	email							    VARCHAR(60),
	pager							    VARCHAR(18),
	address1						    VARCHAR(100),
	address2						    VARCHAR(100),
	address3						    VARCHAR(100),
	address4						    VARCHAR(100),
	address5						    VARCHAR(100),
	address6						    VARCHAR(100)
) ;
CREATE UNIQUE INDEX contact_name ON contact(contact_name);

-- contact, contactgroup:
DROP TABLE IF EXISTS contact_contactgroup;
CREATE TABLE contact_contactgroup(
	contact			INT NOT NULL,
	contactgroup	INT NOT NULL
) ;


DROP TABLE IF EXISTS contactgroup;
CREATE TABLE contactgroup(
	id					INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	contactgroup_name	VARCHAR(75) NOT NULL,
	alias				VARCHAR(160) NOT NULL
) ;
CREATE UNIQUE INDEX contactgroup_name ON contactgroup(contactgroup_name);

DROP TABLE IF EXISTS timeperiod;
CREATE TABLE timeperiod(
	id						INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	timeperiod_name			VARCHAR(75) NOT NULL,
	alias					VARCHAR(160) NOT NULL,
	sunday					VARCHAR(50),
	monday					VARCHAR(50),
	tuesday					VARCHAR(50),
	wednesday				VARCHAR(50),
	thursday				VARCHAR(50),
	friday					VARCHAR(50),
	saturday				VARCHAR(50)
) ;
CREATE UNIQUE INDEX timeperiod_name ON timeperiod(timeperiod_name);

-- junction table for timeperiod<->exclude
DROP TABLE IF EXISTS timeperiod_exclude;
CREATE TABLE timeperiod_exclude(
	timeperiod	INT NOT NULL,
	exclude		INT NOT NULL
) ;


DROP TABLE IF EXISTS command;
CREATE TABLE command(
	id				INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	command_name	VARCHAR(75) NOT NULL,
	command_line	BLOB NOT NULL
) ;
CREATE UNIQUE INDEX command_name ON command(command_name);


-- host table
DROP TABLE IF EXISTS host;
CREATE TABLE host(
	id								INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	host_name						VARCHAR(75),
	alias							VARCHAR(100) NOT NULL,
	display_name					VARCHAR(100),
	address							VARCHAR(75) NOT NULL,
	initial_state					VARCHAR(18),
	check_command					INT,
	check_command_args				TEXT,
	max_check_attempts				SMALLINT,
	check_interval					SMALLINT,
	retry_interval					SMALLINT,
	active_checks_enabled			BOOL,
	passive_checks_enabled			BOOL,
	check_period					INT,
	obsess_over_host				BOOL,
	check_freshness					BOOL,
	freshness_threshold				FLOAT,
	event_handler					INT,
	event_handler_args				TEXT,
	event_handler_enabled			BOOL,
	low_flap_threshold				FLOAT,
	high_flap_threshold				FLOAT,
	flap_detection_enabled			BOOL,
	flap_detection_options			VARCHAR(18),
	process_perf_data				BOOL,
	retain_status_information 		BOOL,
	retain_nonstatus_information	BOOL,
	notification_interval			MEDIUMINT,
	first_notification_delay		INT,
	notification_period				INT,
	notification_options			VARCHAR(15),
	notifications_enabled			BOOL,
	stalking_options				VARCHAR(15),
	notes				VARCHAR(255),
	notes_url			VARCHAR(255),
	action_url			VARCHAR(255),
	icon_image			VARCHAR(60),
	icon_image_alt		VARCHAR(60),
	statusmap_image		VARCHAR(60),
	2d_coords			VARCHAR(20),
	failure_prediction_enabled		BOOL
) ;
CREATE UNIQUE INDEX host_name ON host(host_name);

-- junctions for host objects
DROP TABLE IF EXISTS host_parents;
CREATE TABLE host_parents(
	host	 INT NOT NULL,
	parents	 INT NOT NULL
) ;

DROP TABLE IF EXISTS host_contact;
CREATE TABLE host_contact(
	host	INT NOT NULL,
	contact	INT NOT NULL
) ;

DROP TABLE IF EXISTS host_contactgroup;
CREATE TABLE host_contactgroup(
	host			INT NOT NULL,
	contactgroup	INT NOT NULL
) ;

DROP TABLE IF EXISTS host_hostgroup;
CREATE TABLE host_hostgroup(
	host		INT NOT NULL,
	hostgroup	INT NOT NULL
) ;


DROP TABLE IF EXISTS hostgroup;
CREATE TABLE hostgroup(
	id 		 			INT NOT NULL UNIQUE PRIMARY KEY,
	hostgroup_name		VARCHAR(75),
	alias				VARCHAR(160),
	notes				VARCHAR(160),
	notes_url			VARCHAR(160),
	action_url			VARCHAR(160)
) ;
CREATE UNIQUE INDEX hostgroup_name ON hostgroup(hostgroup_name);


DROP TABLE IF EXISTS service;
CREATE TABLE service(
	id								INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	host_name						INT NOT NULL,
	service_description				VARCHAR(160) NOT NULL,
	display_name					VARCHAR(160),
	is_volatile						BOOL,
	check_command					INT,
	check_command_args				TEXT,
	initial_state					VARCHAR(1),
	max_check_attempts				SMALLINT,
	check_interval					SMALLINT,
	retry_interval					SMALLINT,
	active_checks_enabled			BOOL,
	passive_checks_enabled			BOOL,
	check_period					INT,
	parallelize_check				BOOL,
	obsess_over_service				BOOL,
	check_freshness					BOOL,
	freshness_threshold				INT,
	event_handler					INT,
	event_handler_args				TEXT,
	event_handler_enabled			BOOL,
	low_flap_threshold				FLOAT,
	high_flap_threshold				FLOAT,
	flap_detection_enabled			BOOL,
	flap_detection_options			VARCHAR(18),
	process_perf_data				BOOL,
	retain_status_information		BOOL,
	retain_nonstatus_information	BOOL,
	notification_interval			INT,
	first_notification_delay		INT,
	notification_period				INT,
	notification_options			VARCHAR(15),
	notifications_enabled			BOOL,
	stalking_options				VARCHAR(15),
	notes							VARCHAR(255),
	notes_url						VARCHAR(255),
	action_url						VARCHAR(255),
	icon_image						VARCHAR(60),
	icon_image_alt					VARCHAR(60),
	failure_prediction_enabled		BOOL
) ;
CREATE UNIQUE INDEX service_id ON service(host_name, service_description);

-- junctions for service objects
DROP TABLE IF EXISTS service_contact;
CREATE TABLE service_contact(
	service	INT NOT NULL,
	contact	INT NOT NULL
) ;

DROP TABLE IF EXISTS service_contactgroup;
CREATE TABLE service_contactgroup(
	service			INT NOT NULL,
	contactgroup	INT NOT NULL
) ;

DROP TABLE IF EXISTS service_servicegroup;
CREATE TABLE service_servicegroup(
	service			INT NOT NULL,
	servicegroup	INT NOT NULL
) ;


DROP TABLE IF EXISTS servicegroup;
CREATE TABLE servicegroup(
	id					INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	servicegroup_name	VARCHAR(75) NOT NULL,
	alias				VARCHAR(160) NOT NULL,
	notes				VARCHAR(160),
	notes_url			VARCHAR(160),
	action_url			VARCHAR(160)
) ;
CREATE UNIQUE INDEX servicegroup_name ON servicegroup(servicegroup_name);


DROP TABLE IF EXISTS servicedependency;
CREATE TABLE servicedependency(
	id								INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	service							INT NOT NULL,
	dependent_service				INT NOT NULL,
	dependency_period				INT,
	inherits_parent					BOOL,
	execution_failure_options		VARCHAR(15),
	notification_failure_options	VARCHAR(15)
) ;
CREATE UNIQUE INDEX service ON servicedependency(service);


DROP TABLE IF EXISTS serviceescalation;
CREATE TABLE serviceescalation(
	id								INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	service							INT NOT NULL,
	first_notification				MEDIUMINT,
	last_notification				MEDIUMINT,
	notification_interval			MEDIUMINT,
	escalation_period				INT,
	escalation_options				VARCHAR(15)
) ;

-- junctions for serviceescalation objects
DROP TABLE IF EXISTS serviceescalation_contact;
CREATE TABLE serviceescalation_contact(
	serviceescalation	INT NOT NULL,
	contact				INT NOT NULL
) ;

DROP TABLE IF EXISTS serviceescalation_contactgroup;
CREATE TABLE serviceescalation_contactgroup(
	serviceescalation	INT NOT NULL,
	contactgroup		INT NOT NULL
) ;


DROP TABLE IF EXISTS hostdependency;
CREATE TABLE hostdependency(
	id								INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	host_name						INT NOT NULL,
	dependent_host_name				INT NOT NULL,
	dependency_period				INT,
	inherits_parent					BOOL,
	execution_failure_options		VARCHAR(15),
	notification_failure_options	VARCHAR(15)
) ;

DROP TABLE IF EXISTS hostescalation;
CREATE TABLE hostescalation(
	id								INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	template						INT,
	host_name						INT NOT NULL,
	first_notification				INT,
	last_notification				INT,
	notification_interval			INT,
	escalation_period				INT,
	escalation_options				VARCHAR(15)
) ;


DROP TABLE IF EXISTS hostescalation_contact;
CREATE TABLE hostescalation_contact(
	hostescalation	INT NOT NULL,
	contact			INT NOT NULL
) ;

DROP TABLE IF EXISTS hostescalation_contactgroup;
CREATE TABLE hostescalation_contactgroup(
	hostescalation		INT NOT NULL,
	contactgroup		INT NOT NULL
) ;


-- custom variables
DROP TABLE IF EXISTS custom_vars;
CREATE TABLE custom_vars(
	obj_type	VARCHAR(30) NOT NULL,
	obj_id		INT NOT NULL,
	variable	VARCHAR(100),
	value		VARCHAR(255)
) ;
-- No single object can have multiple variables named the same
CREATE UNIQUE INDEX objvar ON custom_vars(obj_type, obj_id, variable);

-- gui <=> webconfig db scheme cross-pollination ends here

-- Obsoleted tables
DROP TABLE IF EXISTS hostextinfo;
DROP TABLE IF EXISTS serviceextinfo;
